package com.eco.plugin.xx.zgydsso.controller;

import com.eco.plugin.xx.zgydsso.config.PluginSimpleConfig;
import com.eco.plugin.xx.zgydsso.db.bean.UserEntity;
import com.eco.plugin.xx.zgydsso.db.controller.DBController;
import com.eco.plugin.xx.zgydsso.utils.*;
import com.fr.decision.authority.data.Authority;
import com.fr.decision.authority.data.CustomRole;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.json.JSONObject;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.GetMapping;
import com.fr.third.springframework.web.bind.annotation.PostMapping;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@LoginStatusChecker(required = false)
public class ControllerSelf {

    @GetMapping(value = "/toOther")
    @ResponseBody
    public void toOther(HttpServletRequest request,HttpServletResponse response) throws IOException {
        String redirect = request.getParameter("redirect");
        FRUtils.FRLogInfo("toOther");
        UserBean user = null;
        try {
            user = FRUserUtils.getCurrentUserBean(request);
        } catch (Exception e) {
            ResponseUtils.failedResponse(response,"获取当前用户失败!");
        }
        String username = user.getUsername();
        UserEntity userEntity = DBController.getByUsername(username);

        String role = Utils.arrayToStr(user.getRoleIds(),",");
        String city = userEntity.getCity();

        redirect += redirect.contains("?") ? "&operid="+username :"?operid="+username;

        redirect+="&role="+role+"&city="+city;

        if(PluginSimpleConfig.getInstance().getNeedToken()){
            String token = Utils.getCookieByKey(request,"fine_auth_token");
            redirect += "&token="+token;
        }
        FRUtils.FRLogInfo("toOther");

        response.sendRedirect(redirect);
    }

    @PostMapping(value = "/validateToken")
    @ResponseBody
    public Map<String,Object> validateToken(HttpServletRequest request,HttpServletResponse response) throws IOException {
        JSONObject param = Utils.getRequestBody(request);
        String token = param.getString("token");
        String username = FRUserUtils.getUsernameFromToken(token);
        JSONObject userinfo = null;
        try {
            userinfo = getInfoByUsername(username);
        } catch (Exception e) {
            ResponseUtils.failedResponse(response,"获取用户信息失败");
        }
        String role = userinfo.getString("role");
        String city = userinfo.getString("city");
        String authrity = userinfo.getString("authrity");

        Map<String,Object> json = new HashMap<>();

        Map<String,String> result = new HashMap<>();
        result.put("username",username);
        result.put("role",role);
        result.put("city",city);
        result.put("catalogue",authrity);

        json.put("code",200);
        json.put("data",result);

        return json;
    }

    @PostMapping(value = "/getUserInfo")
    @ResponseBody
    public Map<String,Object> getUserInfo(HttpServletRequest request,HttpServletResponse response) throws IOException {
        JSONObject param = Utils.getRequestBody(request);
        String username = param.getString("username");
        JSONObject userinfo = null;
        try {
            userinfo = getInfoByUsername(username);
        } catch (Exception e) {
            ResponseUtils.failedResponse(response,"获取用户信息失败");
        }
        String role = userinfo.getString("role");
        String city = userinfo.getString("city");
        String authrity = userinfo.getString("authrity");

        Map<String,Object> json = new HashMap<>();

        Map<String,String> result = new HashMap<>();
        result.put("username",username);
        result.put("role",role);
        result.put("city",city);
        result.put("catalogue",authrity);

        json.put("code",200);
        json.put("data",result);

        return json;
    }

    private static JSONObject getInfoByUsername(String username) throws Exception {
        JSONObject result = new JSONObject();

        UserBean user = FRUserUtils.getUserBeanByUserName(username);

        List<CustomRole> roleList = FRRoleUtils.getRoleByUser(user.getId());

        String role = "";
        String authrity = "";

        if(roleList != null){
            for(CustomRole cr : roleList){
                role += ","+cr.getId();
                List<Authority> authorityList = AuthorityUtils.getAuthorityByRole(cr.getId());
                authrity+=getAuthorityStr(authorityList);
            }

            if(Utils.isNotNullStr(role)){
                role = role.substring(1);
            }

            if(Utils.isNotNullStr(authrity)){
                authrity = authrity.substring(1);
            }
        }

        result.put("authrity",authrity);



        result.put("role",role);

        UserEntity userEntity = DBController.getByUsername(username);
        String city = userEntity.getCity();

        if(Utils.isNullStr(city)){
            city = "";
        }

        result.put("city",city);



        return result;
    }

    private static String getAuthorityStr(List<Authority> authorityList){
        if(authorityList == null || authorityList.size() <= 0){
            return "";
        }

        String result = "";

        for(Authority authority : authorityList){
            result+= ","+authority.getDisplayName();
        }

        return result;
    }
}
