package com.eco.plugin.xx.zgydsso.db.controller;

import com.eco.plugin.xx.zgydsso.db.bean.RoleUserLinkEntity;
import com.eco.plugin.xx.zgydsso.db.bean.TimeEntity;
import com.eco.plugin.xx.zgydsso.db.bean.UserEntity;
import com.eco.plugin.xx.zgydsso.db.dao.FilenameDao;
import com.eco.plugin.xx.zgydsso.db.dao.RoleUserLinkDao;
import com.eco.plugin.xx.zgydsso.db.dao.UserDao;
import com.eco.plugin.xx.zgydsso.utils.FRUtils;
import com.fr.db.fun.impl.AbstractDBAccessProvider;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.db.dao.DAOProvider;

import java.util.ArrayList;
import java.util.List;

/**
 * @author xx
 * @version 10.0
 * Created by xx on 2021-07-29
 **/
@EnableMetrics
@FunctionRecorder
public class DBController extends AbstractDBAccessProvider {

    private static DBAccessor accessor;

    public static DBAccessor getAccessor() {
        return accessor;
    }

    @Override
    public DAOProvider[] registerDAO() {
        return new DAOProvider[]{
                UserDao.DAO, RoleUserLinkDao.DAO,FilenameDao.DAO
        };
    }

    @Override
    public void onDBAvailable(DBAccessor accessor) {
        DBController.accessor = accessor;
    }


    /**
     * 批量操作
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean batchUser(List<UserEntity> addOrUpdate, List<UserEntity> delete){
        //新增或者删除
        if(addOrUpdate != null && addOrUpdate.size() > 0){
            for(final UserEntity dbe : addOrUpdate){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            UserEntity ae =context.getDAO(UserDao.class).getById(dbe.getId());
                            if(ae != null ){
                                context.getDAO(UserDao.class).update(dbe);
                            }else{
                                context.getDAO(UserDao.class).add(dbe);
                            }
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch addOrUpdate exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        if(delete != null && delete.size() > 0){
            for(final UserEntity dbe : delete){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            context.getDAO(UserDao.class).remove(dbe.getId());
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch delete exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        return true;
    }


    /**
     * 根据username获取信息
     * @param username
     * @return
     */
    public static UserEntity getByUsername(final String username){
        try{
            return accessor.runQueryAction(new DBAction<UserEntity>() {
                @Override
                public UserEntity run(DAOContext context) throws Exception {
                    UserEntity result = context.getDAO(UserDao.class).getByUsername(username);

                    return result == null ? new UserEntity() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getByUsername:"+e.getMessage());
        }

        return new UserEntity();
    }

    /**
     * 批量操作
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean batchLink(List<RoleUserLinkEntity> addOrUpdate, List<RoleUserLinkEntity> delete){
        //新增或者删除
        if(addOrUpdate != null && addOrUpdate.size() > 0){
            for(final RoleUserLinkEntity dbe : addOrUpdate){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            RoleUserLinkEntity ae =context.getDAO(RoleUserLinkDao.class).getById(dbe.getId());
                            if(ae != null ){
                                context.getDAO(RoleUserLinkDao.class).update(dbe);
                            }else{
                                context.getDAO(RoleUserLinkDao.class).add(dbe);
                            }
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch addOrUpdateLink exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        if(delete != null && delete.size() > 0){
            for(final RoleUserLinkEntity dbe : delete){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            context.getDAO(RoleUserLinkDao.class).remove(dbe.getId());
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch delete exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        return true;
    }


    public static boolean addTime(TimeEntity time){

        try{
            accessor.runDMLAction(new DBAction<Boolean>() {
                @Override
                public Boolean run(DAOContext context) throws Exception {
                    context.getDAO(FilenameDao.class).add(time);
                    return true;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("addTime exception ->"+e.getMessage()+time.toString());
        }

        return true;
    }

    public static List<TimeEntity> getAllTime(final String type){
        try{
            return accessor.runQueryAction(new DBAction<List<TimeEntity>>() {
                @Override
                public List<TimeEntity> run(DAOContext context) throws Exception {
                    List<TimeEntity> result = context.getDAO(FilenameDao.class).getAllTime(type);

                    return result == null ? new ArrayList<TimeEntity>() : result;
                }
            });
        }catch(Throwable e){
            FRUtils.FRLogError("exception getAllTime:"+e.getMessage());
        }

        return new ArrayList<TimeEntity>();
    }
}
